/**
  *****************************************************************************
  * @title		can.c
  * @platform	STM32F103
  * @author		Dmitry Ovcharenko (https://www.smartmode.info/)
  * @version	V1.0.0
  * @date		20.06.2016
  *
  * @brief		 CAN - , -
  *
  *******************************************************************************
  * @attention
  *
  *
  * <h2><center>&copy; COPYRIGHT 2016 SmartMODE</center></h2>
*/


#include "stm32f10x.h"
#include "stm32f10x_gpio.h"
#include "stm32f10x_rcc.h"
#include "stm32f10x_can.h"
#include "misc.h"

#include "can.h"


/**
  * @brief    -, CAN
  * @note          STM32F103.
  * @param  None
  * @retval None
  */
void init_CAN(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	/* CAN GPIOs configuration */
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE); 		//   AFIO
	RCC_APB2PeriphClockCmd(CAN1_Periph, ENABLE); 				//   

	RCC_APB1PeriphClockCmd(RCC_APB1Periph_CAN1, ENABLE);		//   CAN-

	//  CAN RX pin
	GPIO_InitStructure.GPIO_Pin   = CAN1_RX_SOURCE;
	GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_IPU;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(CAN1_GPIO_PORT, &GPIO_InitStructure);

	//  CAN TX pin
	GPIO_InitStructure.GPIO_Pin   = CAN1_TX_SOURCE;
	GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(CAN1_GPIO_PORT, &GPIO_InitStructure);

	#ifdef CAN1_ReMap
		GPIO_PinRemapConfig(GPIO_Remap1_CAN1, ENABLE);			//  Can1  PB8, PB9
	#endif

	//  
	CAN_InitTypeDef CAN_InitStructure;

	CAN_DeInit( CAN1);
	CAN_StructInit(&CAN_InitStructure);

	// CAN cell init
	CAN_InitStructure.CAN_TTCM = DISABLE;
	CAN_InitStructure.CAN_ABOM = DISABLE;
	CAN_InitStructure.CAN_AWUM = ENABLE;
	CAN_InitStructure.CAN_NART = DISABLE;
	CAN_InitStructure.CAN_RFLM = DISABLE;
	CAN_InitStructure.CAN_TXFP = DISABLE;
	CAN_InitStructure.CAN_Mode = CAN_Mode_Normal;			//    
//	CAN_InitStructure.CAN_Mode = CAN_Mode_LoopBack;	//      
	CAN_InitStructure.CAN_SJW = CAN_SJW_1tq;
	CAN_InitStructure.CAN_BS1 = CAN_BS1_13tq;
	CAN_InitStructure.CAN_BS2 = CAN_BS2_2tq;
	CAN_InitStructure.CAN_Prescaler = CAN1_SPEED_PRESCALE;	//   
	CAN_Init(CAN1, &CAN_InitStructure);

	// CAN filter init
	CAN_FilterInitTypeDef CAN_FilterInitStructure;
	CAN_FilterInitStructure.CAN_FilterNumber = 1;
	CAN_FilterInitStructure.CAN_FilterMode = CAN_FilterMode_IdMask;
	CAN_FilterInitStructure.CAN_FilterScale = CAN_FilterScale_32bit;
	CAN_FilterInitStructure.CAN_FilterIdHigh = 0x0000;
	CAN_FilterInitStructure.CAN_FilterIdLow = 0x0000;
	CAN_FilterInitStructure.CAN_FilterMaskIdHigh = 0x0000;
	CAN_FilterInitStructure.CAN_FilterMaskIdLow = 0x0000;
	CAN_FilterInitStructure.CAN_FilterFIFOAssignment = CAN_FIFO0;
	CAN_FilterInitStructure.CAN_FilterActivation = ENABLE;
	CAN_FilterInit(&CAN_FilterInitStructure);

	// CAN FIFO0 message pending interrupt enable
	CAN_ITConfig(CAN1, CAN_IT_FMP0, ENABLE);

	// NVIC Configuration
	// Enable CAN1 RX0 interrupt IRQ channel
	NVIC_InitTypeDef NVIC_InitStructure;
	NVIC_InitStructure.NVIC_IRQChannel = USB_LP_CAN1_RX0_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

}

/**
  * @brief    CAN 
  * @note   None
  * @param  None
  * @retval None
  */
void USB_LP_CAN1_RX0_IRQHandler(void)
{
	CanRxMsg RxMessage;

	//   
	RxMessage.DLC = 	0x00;
	RxMessage.ExtId = 	0x00;
	RxMessage.FMI = 	0x00;
	RxMessage.IDE = 	0x00;
	RxMessage.RTR = 	0x00;
	RxMessage.StdId = 	0x00;
	RxMessage.Data [0] = 0x00;
	RxMessage.Data [1] = 0x00;
	RxMessage.Data [2] = 0x00;
	RxMessage.Data [3] = 0x00;
	RxMessage.Data [4] = 0x00;
	RxMessage.Data [5] = 0x00;
	RxMessage.Data [6] = 0x00;
	RxMessage.Data [7] = 0x00;

	if (CAN_GetITStatus(CAN1, CAN_IT_FMP0) != RESET)		//   
	{
		CAN_Receive(CAN1, CAN_FIFO0, &RxMessage);

		//    
		if (RxMessage.IDE == CAN_Id_Standard)
		{
			if (RxMessage.StdId == CAN_CMD_Test_Send)		//    ,       
			{
				CAN_Send_Ok();
			}
			if (RxMessage.StdId == CAN_CMD_Test_Ok)			//    
			{
				//    ,   
			}
		}
	}
}

/**
  * @brief      
  * @note      
  * @param  None
  * @retval None
  */
void CAN_Send_Test(void)
{
	CanTxMsg TxMessage;
	TxMessage.StdId = CAN_CMD_Test_Send;			//  

	TxMessage.ExtId = 0x00;							//     

	TxMessage.IDE = CAN_Id_Standard;				//  
	TxMessage.RTR = CAN_RTR_DATA;					//  
	TxMessage.DLC = 3;								//    3 -   

	TxMessage.Data[0] = 0x00;						//   1
	TxMessage.Data[1] = 0x01;						//   2
	TxMessage.Data[2] = 0x02;						//   3

	CAN_Transmit(CAN1, &TxMessage);
}


/**
  * @brief        
  * @note       
  * @param  None
  * @retval None
  */
void CAN_Send_Ok(void)
{
	CanTxMsg TxMessage;
	TxMessage.StdId = CAN_CMD_Test_Ok;				//  

	TxMessage.ExtId = 0x00;							//     

	TxMessage.IDE = CAN_Id_Standard;				//  
	TxMessage.RTR = CAN_RTR_DATA;					//  
	TxMessage.DLC = 0;								//    0 -   

	CAN_Transmit(CAN1, &TxMessage);
}

